//+------------------------------------------------------------------+
//|                                                     RollOver.mq4 |
//|                                                               AD |
//|                                                scriptong@mail.ru |
//+------------------------------------------------------------------+
#property copyright "AD"
#property link      "scriptong@mail.ru"

extern string TicketsFileName     = "Ticket.dat";
extern string TransactionsFileName     = "Transactions.csv";
extern double Lots = 0.1;
extern bool   CheckRollOver = False;   //       
extern int    MaxOrders    = 100;     //      


datetime OpenTimePos[], CloseTimePos[];
int TicketPos[], TotalOrd;
datetime LastRefresh = 0, DayBegin;

int init()
  {
//----
   TotalOrd = MathMax(MaxOrders, 1);
// --    MaxOrders   
   ArrayResize(TicketPos, TotalOrd);
//----
   ArrayInitialize(TicketPos, -1);
   
//     ,      
  int handle = FileOpen(TicketsFileName, FILE_BIN|FILE_READ);
  if(handle > 0)
    {
    //     
      LastRefresh = FileReadInteger(handle, LONG_VALUE);
    //     
      int i = FileReadInteger(handle, LONG_VALUE);
    //      MaxOrders     ,   
      TotalOrd = MathMax(i, TotalOrd);
      ArrayResize(TicketPos, TotalOrd);
      int k = 0;
    //     ,     
      while(k < i && !FileIsEnding(handle))
        TicketPos[k] = FileReadInteger(handle, LONG_VALUE);
    //  
      FileClose(handle);    
     }
// ----  
   return(0);
  }

//+------------------------------------------------------------------+
//|   /                          |
//+------------------------------------------------------------------+
void FindCloseOrders()
{
// ---   
 for(int i = 0; i < TotalOrd; i++)
  if(TicketPos[i] != -1)
   {
     if(OrderSelect(TicketPos[i], SELECT_BY_TICKET))
      if(OrderCloseTime() != 0)
     //    ,      
       {
         WriteCloseInFile();
         TicketPos[i] = -1;
        }
    } 
// ---
 }
 

//     Excel -          
string DoubleToExcel(double Num, int Precision)
{
  string S = DoubleToStr(Num, Precision);
  int index = StringFind(S, ".", 0);
  if(index > -1)
    return(StringSetChar(S, index, ','));
  return(S);  
 } 

//+------------------------------------------------------------------+
//|                     |
//+------------------------------------------------------------------+
void WriteCloseInFile()
{
// ---   
   int handle = FileOpen(TransactionsFileName, FILE_BIN|FILE_WRITE|FILE_READ);
   if(handle < 1)
    {
      handle = FileOpen(TransactionsFileName, FILE_BIN|FILE_WRITE);
      if(handle < 1)
       {
         Print("    ",TransactionsFileName,"  !");
         return(0);
        }
     }
    else
     {
       int size = FileSize(handle);              
       FileSeek(handle, size, SEEK_SET);
       if(size == 0)
        {
          string S = " ;;"+CharToStr(13)+CharToStr(10);
          FileWriteString(handle, S, StringLen(S));  
         }
      } 
// ---            
   string Writing = TimeToStr(OrderCloseTime())+"; "+DoubleToExcel(OrderSwap(), 2)+"; "
                    +DoubleToExcel(OrderProfit(), 2)+CharToStr(13)+CharToStr(10);   
   FileWriteString(handle, Writing, StringLen(Writing));
   FileClose(handle);
   return(0);
 }

//+------------------------------------------------------------------+
//|                                |
//+------------------------------------------------------------------+
void WriteTicketsInFile()
{
// ---   
  int handle = FileOpen(TicketsFileName, FILE_BIN|FILE_WRITE);
  if(handle < 1)
    {
      Print("    ",TicketsFileName,"  !");
      return(-1);
     }
// ---        
   LastRefresh = TimeCurrent();
   FileWriteInteger(handle, LastRefresh, LONG_VALUE); //    
   FileWriteInteger(handle, TotalOrd, LONG_VALUE);  //    
   for(int i = 0; i < TotalOrd; i++)
    FileWriteInteger(handle, TicketPos[i], LONG_VALUE);
   FileClose(handle);
   return(0);
 }

//+------------------------------------------------------------------+
//|  /                                          |
//+------------------------------------------------------------------+
void OpenOrder(string sym, int cmd, double Vol, double Price, int Slip, double SL, double TP, string Com, int Mag, datetime exp, color arr)
{
 if(!IsTradeContextBusy() && AccountFreeMarginCheck(sym, cmd, Vol) > 0) 
  {
   int ticket = OrderSend(sym, cmd, Vol, Price, Slip, SL, TP, Com, Mag, exp, arr);
   if(ticket > 0)
    {
     //    / ,      
      int i = 0;
      while(i < TotalOrd && TicketPos[i] != -1)
       i++;
      if(TicketPos[i] == -1)
        TicketPos[i] = ticket;
    //    
      WriteTicketsInFile();     
     }
   }
 }

//+-------------------------------------------------------------------+
//|        |
//+-------------------------------------------------------------------+
bool CheckOrders(int Type)
{
 
 bool Result = True;
 int i = 0;
 while(i < TotalOrd)
  {
   if(TicketPos[i] != -1)
    if(OrderSelect(TicketPos[i], SELECT_BY_TICKET))
      if(OrderType() == Type)
        {
          if(Type == OP_BUY  && OrderCloseTime() == 0)
           {  
             if(!IsTradeContextBusy())       
              if(!OrderClose(OrderTicket(), OrderLots(), Bid, 0))
                Result=False;
               else
                RefreshRates();
            }
          if(Type==OP_SELL && OrderCloseTime() == 0)
           { 
             if(!IsTradeContextBusy())       
              if(!OrderClose(OrderTicket(),OrderLots(),Ask,0))
                 Result=False;
               else
                 RefreshRates();
            } 
         } 
       else 
        {
          Result=False;
          break;
         } 
    i++;   
    }
  return(Result); 
}

//+-------------------------------------------------------------------------+
//|          |
//+-------------------------------------------------------------------------+
void RollOver()
{
  for(int i = 0; i < TotalOrd; i++)
   if(TicketPos[i] != -1)
    { 
      if(OrderSelect(i, SELECT_BY_TICKET))
       if(OrderCloseTime() != 0) //         ?
         {
           double SL = OrderStopLoss();
           double TP = OrderTakeProfit();
           bool Res = False;
           for(int k = 0; k < OrdersTotal(); k++)
            if(OrderSelect(k, SELECT_BY_POS, MODE_TRADES))
         //   ,     ,            
             if(OrderOpenTime() > DayBegin && OrderStopLoss() == SL && OrderTakeProfit() == TP)
               {
                TicketPos[i] = OrderTicket();
                Res = True;
                }
           if(!Res)
             TicketPos[i] = -1; //   . ,           
          }
     }
  //      
  WriteTicketsInFile();  
 }


int start()
{

 DayBegin = iTime(Symbol(), PERIOD_D1, 0); 
 //         ,      
 if(LastRefresh < DayBegin && CheckRollOver)
  RollOver();

 double MACD1 = iMACD(Symbol(), PERIOD_H4, 12, 26, 9, PRICE_CLOSE, MODE_MAIN, 1);
 double MACD2 = iMACD(Symbol(), PERIOD_H4, 12, 26, 9, PRICE_CLOSE, MODE_MAIN, 2);

 //  
 if(MACD1 < 0 && MACD2 > 0)
  if(CheckOrders(OP_SELL))
    OpenOrder(Symbol(), OP_BUY, Lots, Ask, 10, 0, 0, NULL, 0, 0, CLR_NONE);
 
 //  
 if(MACD1 > 0 && MACD2 < 0)
  if(CheckOrders(OP_BUY))
    OpenOrder(Symbol(), OP_SELL, Lots, Bid, 10, 0, 0, NULL, 0, 0, CLR_NONE);
    
 FindCloseOrders();   

 return(0);
 }


